'use strict';

exports.__esModule = true;
exports.defaultTemplates = undefined;
exports.default = createNodeFilter;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _strMap = require('../utils/strMap');

var _strMap2 = _interopRequireDefault(_strMap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultTemplates = exports.defaultTemplates = ['default', 'default-{instance}', '{deployment}', '{deployment}-{instance}', '{hostname}', '{hostname}-{instance}', '{hostname}-{deployment}', '{hostname}-{deployment}-{instance}', 'local', 'local-{instance}', 'local-{deployment}', 'local-{deployment}-{instance}'];

function subStrFrom(str, fromStr) {
    return str.substring(str.indexOf(fromStr) + 1);
}

function createNodeFilter(_ref) {
    var _ref$instance = _ref.instance,
        instance = _ref$instance === undefined ? 'main' : _ref$instance,
        _ref$hostname = _ref.hostname,
        hostname = _ref$hostname === undefined ? _os2.default.hostname() : _ref$hostname,
        _ref$tagSeparator = _ref.tagSeparator,
        tagSeparator = _ref$tagSeparator === undefined ? '#' : _ref$tagSeparator,
        _ref$env = _ref.env,
        env = _ref$env === undefined ? process.env.NODE_ENV || 'production' : _ref$env,
        _ref$templates = _ref.templates,
        templates = _ref$templates === undefined ? defaultTemplates : _ref$templates;

    var mapTokens = {
        hostname: hostname,
        deployment: env,
        instance: instance
    };
    var filesTemplates = templates.map(function (template) {
        return (0, _strMap2.default)(template, mapTokens);
    });

    function normalize(relFile) {
        return subStrFrom(_path2.default.basename(relFile, _path2.default.extname(relFile)), tagSeparator);
    }

    return function nodeFilter(relFiles) {
        return filesTemplates.reduce(function (acc, template) {
            return acc.concat(relFiles.filter(function (relFile) {
                return normalize(relFile) === template;
            }));
        }, []);
    };
}
//# sourceMappingURL=createNodeFilter.js.map